pub struct TVOr<Tag, C, Rest>(pub Tag, pub C, pub Rest);Expand description
One of the dependent family of combinators
Typically used to build linear chains of tagged unions: TVOr(0x01u8, fmt1, TVOr(0x02u8, fmt2, Uninhabited()))
Tuple Fields
0: Tag1: C2: RestTrait Implementations
Sourceimpl<Tag, C, Rest> DepCombinator for TVOr<Tag, C, Rest>where
C: Consistency,
Rest: DepCombinator<Key = Tag>,
impl<Tag, C, Rest> DepCombinator for TVOr<Tag, C, Rest>where
C: Consistency,
Rest: DepCombinator<Key = Tag>,
Sourceopen spec fn apply(&self, key: Self::Key) -> Self::Body
open spec fn apply(&self, key: Self::Key) -> Self::Body
{ Choice(Cond(key == self.0, self.1), self.2.apply(key)) }Sourceopen spec fn recover(&self, value: Self::Val) -> Self::Key
open spec fn recover(&self, value: Self::Val) -> Self::Key
{
match value {
Sum::Inl(_) => self.0,
Sum::Inr(vr) => self.2.recover(vr),
}
}Sourceopen spec fn recover_inv(&self) -> bool
open spec fn recover_inv(&self) -> bool
{ self.2.recover_inv() }Sourceproof fn lemma_recover_consistent(&self, key: Self::Key, value: Self::Val)
proof fn lemma_recover_consistent(&self, key: Self::Key, value: Self::Val)
Sourcetype Key = Tag
type Key = Tag
The type of keys parsed by the head combinator/to be recovered during serialization.
Sourcetype Val = Sum<<C as Consistency>::Val, <Rest as DepCombinator>::Val>
type Val = Sum<<C as Consistency>::Val, <Rest as DepCombinator>::Val>
The type of values consumed/produced by the body combinator.